#include <opencv2/core/core.hpp>
#include <opencv2/highgui/highgui.hpp>
#include <iostream>
#include <stdio.h>
#include "7seg.hpp"

using namespace cv;
using namespace std;

#define Q_C 113	// q key on keyboard
#define A_C 97	// a key
#define O_C 111	// o key
#define L_C 108	// l key
#define S_C 115	// s key
#define ESC 27	// esc key
#define TILE_WIDTH 10	// width of tile
#define TILE_HEIGHT 60	// height of tile
#define FINGER_WIDTH 30	// width of finger detection field
#define	IMG_WIDTH 1024	// image width
#define	IMG_HEIGHT 699	// image height
#define BALL_RADIUS 8	// ball radius

// 1024 x 699

int dir_change(Point pt, int direction, Point p1_start, Point p1_end, Point p2_start, Point p2_end);
void printScreen(Mat image, Point p1_start, Point p1_end, Point p2_start, Point p2_end, Point ball_center, int key_pressed);
void initializeScreen(Mat image, Point p1_start, Point p1_end, Point p2_start, Point p2_end, Point ball_center);
void showScore(Mat image);


//			   0   1   2   3   4    5    6    7   8   9  10  11  12  13  14  15
int x_dir[] = {1,  1,  1,  1,  0,  -1,  -1,  -1, -1, -1, -1, -1,  0,  1,  1,  1}; // ball movement on x axis
int y_dir[] = {0, -1, -1, -1,  0,  -1,  -1,  -1,  0,  1,  1,  1,  1,  1,  1,  1}; // ball movement on y axis
int cnt = 0;
int reset = 0;	
int score[] = {0, 0};
const int lineType = 8;   
const int img_half_width = IMG_WIDTH/2;
const int img_half_height = IMG_HEIGHT/2;
const int tile_half_height = TILE_HEIGHT/2;
const int tile_third = TILE_HEIGHT/3;
const Scalar white = Scalar(255, 255, 255); // RBG code for white colour
const Scalar black = Scalar(0, 0, 0);		// RGB code for black colour
const Scalar purple = Scalar(123, 65, 100);	// RGB code for purple colour

// score segments for player 1
const Point a1_start = Point(img_half_width-40, img_half_height-35);
const Point a1_end = Point(img_half_width-70, img_half_height-45);
const Point b1_start = Point(img_half_width-40, img_half_height-35);
const Point b1_end = Point(img_half_width-30, img_half_height-5);
const Point c1_start = Point(img_half_width-40, img_half_height+35);
const Point c1_end = Point(img_half_width-30, img_half_height+5);
const Point d1_start = Point(img_half_width-40, img_half_height+35);
const Point d1_end = Point(img_half_width-70, img_half_height+45);
const Point e1_start = Point(img_half_width-80, img_half_height+35);
const Point e1_end = Point(img_half_width-70, img_half_height+5);
const Point f1_start = Point(img_half_width-80, img_half_height-35);
const Point f1_end = Point(img_half_width-70, img_half_height-5);
const Point g1_start = Point(img_half_width-40, img_half_height-5);
const Point g1_end = Point(img_half_width-70, img_half_height+5);

// score segments for player 2
const Point a2_start = Point(img_half_width+40, img_half_height-35);
const Point a2_end = Point(img_half_width+70, img_half_height-45);
const Point b2_start = Point(img_half_width+80, img_half_height-35);
const Point b2_end = Point(img_half_width+70, img_half_height-5);
const Point c2_start = Point(img_half_width+80, img_half_height+35);
const Point c2_end = Point(img_half_width+70, img_half_height+5);
const Point d2_start = Point(img_half_width+40, img_half_height+35);
const Point d2_end = Point(img_half_width+70, img_half_height+45);
const Point e2_start = Point(img_half_width+40, img_half_height+35);
const Point e2_end = Point(img_half_width+30, img_half_height+5);
const Point f2_start = Point(img_half_width+40, img_half_height-35);
const Point f2_end = Point(img_half_width+30, img_half_height-5);
const Point g2_start = Point(img_half_width+40, img_half_height-5);
const Point g2_end = Point(img_half_width+70, img_half_height+5);

// function that calculates if direction change is needed
int dir_change(Point pt, int direction, Point p1_start, Point p1_end, Point p2_start, Point p2_end){
	int dir = direction;
	int ball_right_edge = pt.x+BALL_RADIUS;
	int ball_left_edge = pt.x-BALL_RADIUS;
	int ball_uper_edge = pt.y-BALL_RADIUS;
	int ball_lover_edge = pt.y+BALL_RADIUS;
	int p1_tile_edge = FINGER_WIDTH+TILE_WIDTH;
	int p2_tile_edge = IMG_WIDTH-FINGER_WIDTH-TILE_WIDTH;
	
	int p1_tile_middle_up = p1_start.y+tile_third;
	int p1_tile_middle_down = p1_tile_middle_up+tile_third;
	int p2_tile_middle_up = p2_start.y+tile_third;
	int p2_tile_middle_down = p2_tile_middle_up+tile_third;
	
	if( (ball_right_edge == p2_tile_edge) &&
	    (pt.y >= p2_start.y) && (pt.y <= p2_end.y) ){
	    
	    cnt = 0;
	
		if(pt.y < p2_tile_middle_up){
			if(direction == 3)
				return 5;
			else if(direction > 12)
				return (23 - direction);
			else
				return (7 - direction);
		}
		
		else if(pt.y > p2_tile_middle_down){
			if(direction == 13)
				return 11;
			else if(direction < 4)
				return (9 - direction);
			else
				return (25 - direction);
		}
		
		else{
			if(direction <= 8)
				return (8 - direction);
			else
				return (24 - direction);  	 
		} 
	}
	
	else if( (ball_left_edge == p1_tile_edge) &&
			 (pt.y >= p1_start.y) && (pt.y <= p1_end.y) ){
			 
		cnt = 0;
		
		if(pt.y < p1_tile_middle_up){
			if(direction == 5)
				return 3;
			else if(direction > 9)
				return (25 - direction);
			else
				return (9 - direction);
		}
		
		else if(pt.y > p1_tile_middle_down){
			if(direction == 11)
				return 11;
			else if(direction > 7)
				return (23 - direction);
			else
				return (7 - direction);
		}
		
		else{
			if(direction <= 8)
				return (8 - direction);
			else
				return (24 - direction); 
		} 
	}
	
	else if((ball_uper_edge == 0) ||
	   		(ball_lover_edge == IMG_HEIGHT)){
	   	
	   	cnt = 0;	
		return (16 - direction);  
		
	}
	
	else if((ball_right_edge > p2_tile_edge) ||
	   		(ball_left_edge < p1_tile_edge)){
	   
	    if(ball_left_edge == FINGER_WIDTH){
	    	score[1] += 1;
	    	cout << score[0] << ":" << score[1] << endl;
	    	reset = 1;
	    	return dir;
	    }
	    else if(ball_right_edge == IMG_WIDTH-FINGER_WIDTH){
	    	score[0] += 1;
	        cout << score[0] << ":" << score[1] << endl;
	        reset = 1;
	        return dir;
	    }
	    
	    else if((ball_lover_edge == p1_start.y) ||
	   	   		(ball_uper_edge == p1_end.y)){
	   	   
	   	    cnt = 0;
	   		return (16 - direction);  
		}
		
		else if((ball_lover_edge == p2_start.y) ||
	   	   (ball_uper_edge == p2_end.y)){
	   	   
	   	    cnt = 0;
	   		return (16 - direction); 
		}
	 
	}
	
	return dir;
}

// function that initialize and restart the screen
void initializeScreen(Mat image, Point p1_start, Point p1_end, Point p2_start, Point p2_end, Point ball_center){

	rectangle(image, Point(FINGER_WIDTH+TILE_WIDTH,0), Point(IMG_WIDTH-FINGER_WIDTH-TILE_WIDTH, IMG_HEIGHT), black, CV_FILLED, lineType, 0);
	circle(image, ball_center, BALL_RADIUS, white, CV_FILLED, lineType, 0);
	rectangle(image, Point(FINGER_WIDTH, 0), Point(FINGER_WIDTH+TILE_WIDTH, IMG_HEIGHT), black, CV_FILLED, lineType, 0);
	rectangle(image, Point(IMG_WIDTH-FINGER_WIDTH, 0), Point(IMG_WIDTH-FINGER_WIDTH-TILE_WIDTH, IMG_HEIGHT), black, CV_FILLED, lineType, 0);
	rectangle(image, p1_start, p1_end,white, CV_FILLED, lineType, 0);   
	rectangle(image, p2_start, p2_end, white, CV_FILLED, lineType, 0);
	rectangle(image, Point(0,0), Point(FINGER_WIDTH-1, IMG_HEIGHT), purple, CV_FILLED, lineType, 0);
	rectangle(image, Point(IMG_WIDTH-FINGER_WIDTH+1,0), Point(IMG_WIDTH, IMG_HEIGHT), purple, CV_FILLED, lineType, 0);
	
	imshow("Display window", image);
	waitKey(1);
}

// function that prints the tiles, ball and score
void printScreen(Mat image, Point p1_start, Point p1_end, Point p2_start, Point p2_end, Point ball_center, int key_pressed){
	
	// draw the ball
	circle(image, ball_center, BALL_RADIUS, white, CV_FILLED, lineType, 0);
	
	// draw the tile which is moved
	if((key_pressed == Q_C) || (key_pressed == A_C))
		rectangle(image, p1_start, p1_end, white, CV_FILLED, lineType, 0);   
	else if((key_pressed == O_C) || (key_pressed == L_C))
		rectangle(image, p2_start, p2_end, white, CV_FILLED, lineType, 0);
		  
	// if somebody scored, show the score   
	if(reset)
		showScore(image);
}

// function for drawing score (the macros are defined in 7seg.hpp)
void showScore(Mat image){
	switch(score[0]){
		case 0: print1_0; break;	
		case 1:	print1_1; break;
		case 2: print1_2; break;
		case 3: print1_3; break;
		case 4:	print1_4; break;
		case 5: print1_5; break;
		case 6: print1_6; break;
		case 7: print1_7; break;
		case 8: print1_8; break;
		case 9: print1_9; break;
	}
	switch(score[1]){
		case 0: print2_0; break;	
		case 1:	print2_1; break;
		case 2: print2_2; break;
		case 3: print2_3; break;
		case 4:	print2_4; break;
		case 5: print2_5; break;
		case 6: print2_6; break;
		case 7: print2_7; break;
		case 8: print2_8; break;
		case 9: print2_9; break;
	}
}

int main(int argc, char ** argv){
	/*if(argc != 2){
		cout<<"Usage: display_image ImageToLoadAndDispaly"<<endl;
	return -1;
	}*/

	Mat image;
	int thickness = 2; 
    int direction = 8;
    int speed = 1;
    int key_pressed = -1;
    
	image = imread("R.png", CV_LOAD_IMAGE_COLOR);
	
	if(!image.data){
		cout<<"Could not open or find the image"<<endl;
		return -1;	
	}
	
	namedWindow("Display window", WINDOW_AUTOSIZE);
	
	int p1_y = img_half_height;
	int p2_y = img_half_height;
	
	// coordinates for player 1 and 2 tiles and ball
	Point p1_start = Point(FINGER_WIDTH, p1_y-tile_half_height);
	Point p1_end = Point(FINGER_WIDTH+TILE_WIDTH, p1_y+tile_half_height);;
	Point p2_start = Point(IMG_WIDTH-FINGER_WIDTH, p2_y-tile_half_height);
	Point p2_end = Point(IMG_WIDTH-FINGER_WIDTH-TILE_WIDTH, p2_y+tile_half_height);
	Point ball_center = Point(img_half_width, img_half_height);
	
	initializeScreen(image, p1_start, p1_end, p2_start, p2_end, ball_center);

	while(1){
		     	   
		if(reset){	// reset ocures when a player scores
			ball_center = Point(img_half_width, img_half_height);
			reset++;
			if(reset == 3){
				reset = 0;
				initializeScreen(image, p1_start, p1_end, p2_start, p2_end, ball_center);
			}
			sleep(1);
		}	
		else{				 	   	
			// erase the ball (draw a black one over it)
			circle(image, ball_center, BALL_RADIUS, black, CV_FILLED, lineType, 0);
			
			// calculate the ball direction
			direction = dir_change(ball_center, direction, p1_start, p1_end, p2_start, p2_end);
			
			// calculate the new ball coordinates
			if((direction == 1) || (direction == 7) || (direction == 9) || (direction == 15)){
				if(cnt == 0){
					ball_center.x += x_dir[direction]*speed;
					ball_center.y += y_dir[direction]*speed;
				}
				else
					ball_center.x += x_dir[direction]*speed;	
	
				cnt++;
				cnt %= 2;
			}
			else if((direction == 3) || (direction == 5) || (direction == 11) || (direction == 13)){
				if(cnt == 0){
					ball_center.y += y_dir[direction]*speed;
					ball_center.x += x_dir[direction]*speed;
				}
				else
					ball_center.y += y_dir[direction]*speed;

				cnt++;		
				cnt %= 2;	
			}
			else{
				ball_center.x += x_dir[direction]*speed;
				ball_center.y += y_dir[direction]*speed;
			}
		
			// calculate the new possition of tile if the key is pressed
			if(key_pressed > 0){
				p1_start.y = p1_y-tile_half_height;
				p1_end.y = p1_y+tile_half_height;
	
				p2_start.y = p2_y-tile_half_height;
				p2_end.y = p2_y+tile_half_height;
			}		
		
			printScreen(image, p1_start, p1_end, p2_start, p2_end, ball_center, key_pressed);

			imshow("Display window", image);
		
			key_pressed = waitKey(1); // waitKey function returns an integer that represents pushed key
			
			// if the tile is moved, erase it (draw a black tile over the existing white)
			if((key_pressed == Q_C) || (key_pressed == A_C))	
				rectangle(image, p1_start, p1_end, black, CV_FILLED, lineType, 0);   
			else if((key_pressed == O_C) || (key_pressed == L_C))
				rectangle(image, p2_start, p2_end, black, CV_FILLED, lineType, 0);
			
			// calculate new coordinates for tiles if the key is pressed
			switch(key_pressed){
				case Q_C:
					if(p1_y-tile_half_height >= 0)
						p1_y -= 3; break;
				
				case A_C:
					if(p1_y+tile_half_height <= IMG_HEIGHT)
						p1_y += 3; break;
				
				case O_C:
					if(p2_y-tile_half_height >= 0)
						p2_y -= 3; break;
				
				case L_C:
					if(p2_y+tile_half_height <= IMG_HEIGHT)
						p2_y += 3; break;
				
				case S_C:
					speed ++; break;
					
				case ESC:
					return 0;		
			}
		}
	}

	return 0;
}
